@@ -20,6 +20,21 @@ def testlines(fid):
         return False
 
 
+def get_nol(b, mdata):
+    """
+    Return the number of lines based on data
+    """
+    nol = [1, 1, 1, 1, 1, 4, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0]
+
+    if b == 5:  # for transformer
+        if mdata[0][2] == 0:  # two-winding transformer
+            return 4
+        else:  # three-widning transf
+            return 5
+
+    return nol[b]
+
+
 def read(file, system):
     """read PSS/E RAW file v32 format"""
 
@@ -29,7 +44,7 @@ def read(file, system):
         'interarea', 'owner', 'facts', 'swshunt', 'gne', 'Q'
     ]
     nol = [1, 1, 1, 1, 1, 4, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0]
-    rawd = re.compile('rawd\d\d')
+    rawd = re.compile(r'rawd\d\d')
 
     retval = True
     version = 0
@@ -53,14 +68,18 @@ def read(file, system):
             mva = float(data[1])
             system.config.mva = mva
             system.config.freq = float(data[5])
-            version = int(data[2])
 
+            # get raw file version
+            version = int(data[2])
             if not version:
                 version = int(rawd.search(line).group(0).strip('rawd'))
+            logger.debug('PSSE raw version {} detected'.format(version))
+
             if version < 32 or version > 33:
                 logger.warning(
                     'RAW file version is not 32 or 33. Error may occur.')
             continue
+
         elif num == 1:  # store the case info line
             logger.info(line)
             continue
@@ -77,8 +96,10 @@ def read(file, system):
         data = [to_number(item) for item in data]
         mdata.append(data)
         mline += 1
-        if mline >= nol[b]:
-            if nol[b] == 1:
+        nol = get_nol(b, mdata)
+
+        if mline >= nol:
+            if nol == 1:
                 mdata = mdata[0]
             raw[blocks[b]].append(mdata)
             mdata = []
@@ -87,14 +108,17 @@ def read(file, system):
 
     # add device elements to system
     sw = {}  # idx:a0
+    max_bus = []
     for data in raw['bus']:
         """version 32:
           0,   1,      2,     3,    4,   5,  6,   7,  8
-          ID, NAME, BasekV, Type, Area Zone Owner Va, Vm
+          ID, NAME, BasekV, Type, Area Zone Owner Vm, Va
         """
         idx = data[0]
+        max_bus.append(idx)
         ty = data[3]
         a0 = data[8] * deg2rad
+
         if ty == 3:
             sw[idx] = a0
         param = {
@@ -108,7 +132,7 @@ def read(file, system):
             'owner': data[6],
         }
         system.Bus.elem_add(**param)
-
+    max_bus = max(max_bus)
     for data in raw['load']:
         """version 32:
           0,  1,      2,    3,    4,    5,    6,      7,   8,  9, 10,   11
@@ -195,48 +219,84 @@ def read(file, system):
         }
         system.Line.elem_add(**param)
 
+    xf_3_count = 1
     for data in raw['transf']:
-        """
-        I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4
-        R1-2,X1-2,SBASE1-2
-        WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD1,CONT1,RMA1,RMI1,VMA1,VMI1,NTP1,TAB1,CR1,CX1
-        WINDV2,NOMV2
-        """
-        if len(data[1]) < 5:
-            ty = 2
-        else:
-            ty = 3
-        if ty == 3:
-            raise NotImplementedError(
-                'Three-winding transformer not implemented')
+        if len(data) == 4:
+            """
+            I,J,K,CKT,CW,CZ,CM,MAG1,MAG2,NMETR,'NAME',STAT,O1,F1,...,O4,F4
+            R1-2,X1-2,SBASE1-2
+            WINDV1,NOMV1,ANG1,RATA1,RATB1,RATC1,COD1,CONT1,RMA1,RMI1,VMA1,VMI1,NTP1,TAB1,CR1,CX1
+            WINDV2,NOMV2
+            """
 
-        tap = data[2][0]
-        phi = data[2][2]
+            tap = data[2][0]
+            phi = data[2][2]
 
-        if tap == 1 and phi == 0:
-            trasf = False
+            if tap == 1 and phi == 0:
+                trasf = False
+            else:
+                trasf = True
+            param = {
+                'trasf': trasf,
+                'bus1': data[0][0],
+                'bus2': data[0][1],
+                'u': data[0][11],
+                'b': data[0][8],
+                'r': data[1][0],
+                'x': data[1][1],
+                'tap': tap,
+                'phi': phi,
+                'rate_a': data[2][3],
+                'Vn': system.Bus.get_field('Vn', data[0][0]),
+                'Vn2': system.Bus.get_field('Vn', data[0][1]),
+            }
+            system.Line.elem_add(**param)
         else:
-            trasf = True
-        param = {
-            'trasf': trasf,
-            'bus1': data[0][0],
-            'bus2': data[0][1],
-            'u': data[0][11],
-            'b': data[0][8],
-            'r': data[1][0],
-            'x': data[1][1],
-            'tap': tap,
-            'phi': phi,
-            'rate_a': data[2][3],
-            'Vn': system.Bus.get_field('Vn', data[0][0]),
-            'Vn2': system.Bus.get_field('Vn', data[0][1]),
-        }
-        system.Line.elem_add(**param)
+            """
+            I, J, K, CKT, CW, CZ, CM, MAG1, MAG2, NMETR, 'NAME', STAT, Ol, Fl,...,o4, F4
+            R1—2, X1—2, SBASE1—2, R2—3, X2—3, SBASE2—3, R3—1, X3—1, SBASE3—1, VMSTAR, ANSTAR
+            WINDV1, NOMV1, ANG1, RATA1, BATB1, RATC1, COD1, CONT1, RMA1, RMI1, VMA1, VMI1, NTP1, TAB1, CR1, CX1
+            WINDV2, NOMV2, ANG2, RATA2, BATB2, RATC2, COD2, CONT2, RMA2, RMI2, VMA2, VMI2, NTP2, TAB2, CR2, CX2
+            WINDV3, NOMV3, ANG3, RATA3, BATB3, RATC3, COD3, CONT3, RMA3, RMI3, VMA3, VMI3, NTP3, TAB3, CR3, CX3
+            """
+            param = {
+                'idx': max_bus+xf_3_count,
+                'name': '_'.join(str(data[0][:3])),
+                'Vn': system.Bus.get_field('Vn', data[0][0]),
+                'voltage': data[1][-2],
+                'angle': data[1][-1]/180*3.14
+            }
+            system.Bus.elem_add(**param)
+
+            r = []
+            x = []
+            r.append((data[1][0] + data[1][6] - data[1][3])/2)
+            r.append((data[1][3] + data[1][0] - data[1][6])/2)
+            r.append((data[1][6] + data[1][3] - data[1][0])/2)
+            x.append((data[1][1] + data[1][7] - data[1][4])/2)
+            x.append((data[1][4] + data[1][1] - data[1][7])/2)
+            x.append((data[1][7] + data[1][4] - data[1][1])/2)
+            for i in range(0, 3):
+                param = {
+                    'trasf': True,
+                    'bus1': data[0][i],
+                    'bus2': max_bus+xf_3_count,
+                    'u': data[0][11],
+                    'b': data[0][8],
+                    'r': r[i],
+                    'x': x[i],
+                    'tap': data[2+i][0],
+                    'phi': data[2+i][2],
+                    'rate_a': data[2+i][3],
+                    'Vn': system.Bus.get_field('Vn', data[0][i]),
+                    'Vn2': system.Bus.get_field('Vn', data[0][0]),
+                }
+                system.Line.elem_add(**param)
+            xf_3_count += 1
 
     for data in raw['swshunt']:
         # I, MODSW, ADJM, STAT, VSWHI, VSWLO, SWREM, RMPCT, ’RMIDNT’,
         # BINIT, N1, B1, N2, B2, ... N8, B8
-
         bus = data[0]
         vn = system.Bus.get_field('Vn', bus)
         param = {
