@@ -13,26 +13,28 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 """
 Base class for building ANDES models
 """
 
 import sys
-from logging import DEBUG, INFO, WARNING, ERROR, CRITICAL  # NOQA
+import importlib
+import numpy as np
+import logging
 
 from cvxopt import matrix, spmatrix  # NOQA
 from cvxopt import mul, div
 
+from logging import DEBUG, INFO, WARNING, ERROR, CRITICAL  # NOQA
+
 from ..utils.math import agtb, altb, index, zeros  # NOQA
 from ..utils.tab import Tab
 
-# pandas import slows down the program startup by 0.3 seconds
-import pandas as pd
-import numpy as np
-
-import logging
 logger = logging.getLogger(__name__)
 
+pd = None
+
 
 class ModelBase(object):
     """base class for power system device models"""
@@ -78,6 +80,16 @@ class ModelBase(object):
         self._fnamex = []
         self._fnamey = []
 
+        # `self.mdl_from`:
+        #    a list of lists, which contain tuples of (model, idx)
+        #    of elements connected to the corresponding element
+        # `self.mdl_to`:
+        #    a list of lists, which contain tuples of (model, idx) of
+        #    elements the corresponding element connects to
+
+        self.mdl_from = []
+        self.mdl_to = []
+
         # parameters to be converted to matrix
         self._params = ['u', 'Sn', 'Vn']
 
@@ -412,6 +424,8 @@ class ModelBase(object):
         :return: field values
         """
         assert astype in (None, list, matrix)
+        ret = None
+
         if idx is None:
             idx = self.idx
 
@@ -424,15 +438,55 @@ class ModelBase(object):
         # =================================================================
 
         uid = self.get_uid(idx)
-        if not astype:
-            astype = type(self.__dict__[field])
 
-        ret = matrix(self.__dict__[field])[uid]
+        field_data = self.__dict__[field]
 
-        if isinstance(idx, (float, int, str)):
-            return ret
+        if isinstance(field_data, matrix):
+            ret = field_data[uid]
+        elif isinstance(field_data, list):
+            if isinstance(idx, (float, int, str)):
+                ret = field_data[uid]
+            else:
+                ret = [field_data[x] for x in uid]
+
+        if astype is not None:
+            ret = astype(ret)
+
+        return ret
+
+    def set_field(self, field, idx, value, sysbase=False):
+        """
+        Set the field of an element to the given value
+
+        Parameters
+        ----------
+        field
+        idx
+        value
+        sysbase
+
+        Returns
+        -------
+
+        """
+        if field not in self.__dict__:
+            logger.error('Field <{}> does not exist.'.format(field))
+            return
+
+        if idx not in self.idx:
+            logger.error('Idx <{}> is not valid'.format(field))
+
+        uid = self.get_uid(idx)
+        old_type = type(self.__dict__[field][uid])
+
+        in_store = True if field in self._store else False
+
+        if (not sysbase) and in_store:
+            self._store[field][uid] = old_type(value)
+            logger.info('<{}> set field <{}> of <{}> to {} in _store'.format(self._name, field, idx, value))
         else:
-            return astype(ret)
+            self.__dict__[field][uid] = old_type(value)
+            logger.info('<{}> set field <{}> of <{}> to {} in class dict'.format(self._name, field, idx, value))
 
     def _alloc(self):
         """
@@ -462,7 +516,7 @@ class ModelBase(object):
         ret = {}
 
         for key in self.data_keys:
-            if sysbase and (key in self._store):
+            if (not sysbase) and (key in self._store):
                 val = self._store[key]
             else:
                 val = self.__dict__[key]
@@ -497,6 +551,17 @@ class ModelBase(object):
         keys = ['idx', 'name', 'u'] + sorted(self._data.keys())
         return set(keys)
 
+    def fxcall_idx(self):
+        """
+        Precompute the row and column indices of the non-zero elements in the df/dx matrix
+
+        Returns
+        -------
+        (row indices, col indices)
+        """
+
+        pass
+
     def data_to_list(self, sysbase=False):
         """
         Return the loaded model data as a list of dictionaries.
@@ -528,7 +593,7 @@ class ModelBase(object):
 
         Parameters
         ----------
-        data: list
+        data : list
             List of parameter dictionaries
 
         Returns
@@ -544,7 +609,7 @@ class ModelBase(object):
 
         Parameters
         ----------
-        data: list
+        data : dict
             List of parameter dictionaries
 
         Returns
@@ -574,7 +639,9 @@ class ModelBase(object):
         """
 
         p_dict_comp = self.data_to_dict(sysbase=sysbase)
-        self.param_df = pd.DataFrame(data=p_dict_comp)
+        self._check_pd()
+
+        self.param_df = pd.DataFrame(data=p_dict_comp).set_index('idx')
 
         return self.param_df
 
@@ -585,6 +652,7 @@ class ModelBase(object):
         :return: pandas.DataFrame
         """
         ret = {}
+        self._check_pd()
 
         if self._flags['address'] is False:
             return pd.DataFrame.from_dict(ret)
@@ -599,7 +667,19 @@ class ModelBase(object):
             idx = self.__dict__[y]
             ret.update({y: self.system.dae.y[idx]})
 
-        return pd.DataFrame.from_dict(ret)
+        var_df = pd.DataFrame.from_dict(ret).set_index('idx')
+
+        return var_df
+
+    @staticmethod
+    def _check_pd():
+        """
+        Import pandas to globals() if not exist
+        """
+        if globals()['pd'] is None:
+            globals()['pd'] = importlib.import_module('pandas')
+
+        return
 
     def param_remove(self, param: 'str') -> None:
         """
@@ -797,6 +877,10 @@ class ModelBase(object):
         self.__dict__[dest] = self.read_data_ext(
             model, field, idx, astype=astype)
 
+        if idx is not None:
+            if len(idx) == self.n:
+                self.link_to(model, idx, self.idx)
+
     def elem_add(self, idx=None, name=None, **kwargs):
         """
         Add an element of this model
@@ -812,6 +896,10 @@ class ModelBase(object):
 
         self.uid[idx] = self.n
         self.idx.append(idx)
+
+        self.mdl_to.append(list())
+        self.mdl_from.append(list())
+
         # self.n += 1
 
         if name is None:
@@ -880,6 +968,8 @@ class ModelBase(object):
         # self.n -= 1
         self.uid.pop(key, '')
         self.idx.pop(item)
+        self.mdl_from.pop(item)
+        self.mdl_to.pop(item)
 
         for x, y in self.uid.items():
             if y > item:
@@ -920,6 +1010,24 @@ class ModelBase(object):
         if convert and self.n:
             self._param_to_matrix()
 
+    def reload_new_param(self):
+        """
+        Reload the new params modified in `self._store` and then convert to sysbase and store in class.
+
+        Returns
+        -------
+
+        """
+        if not self._flags['sysbase']:  # parameters have not been converted to sys base
+            self.data_to_sys_base()
+        else:  # parameters are in sys base. `self._store` has the parameters in elem base
+
+            for key, val in self._store:
+                self.__dict__[key] = val
+            self._flags['sysbase'] = False
+            self.data_to_sys_base()
+            logger.debug('Reloaded system parameter for _store.')
+
     def data_to_sys_base(self):
         """
         Converts parameters to system base. Stores a copy in ``self._store``.
@@ -1004,7 +1112,7 @@ class ModelBase(object):
         if self._flags['sysbase'] is False:
             return
 
-        for key, val in self._store:
+        for key, val in self._store.items():
             self.__dict__[key] = val
 
         self._flags['sysbase'] = False
@@ -1089,10 +1197,12 @@ class ModelBase(object):
 
         """
         group_by = self._config['address_group_by']
-
-        assert not self._flags['address']
         assert group_by in ('element', 'variable')
 
+        if self._flags['address'] is True:
+            logger.debug('{} addresses exist. Skipped.'.format(self._name))
+            return
+
         m0 = self.system.dae.m
         n0 = self.system.dae.n
         mend = m0 + len(self._algebs) * self.n
@@ -1141,18 +1251,18 @@ class ModelBase(object):
                 fnamex = self._fnamex[e]
                 idx = self.__dict__[var][i]
 
-                varname.unamex[idx] = '{} {}'.format(unamex, iname)[:24]
-                varname.fnamex[idx] = '$' + '{}\ {}'.format(
-                    fnamex, iname.replace(' ', '\\ '))[:24] + '$'
+                varname.unamex[idx] = '{} {}'.format(unamex, iname)[:33]
+                varname.fnamex[idx] = '$' + r'{}\ {}'.format(
+                    fnamex, iname.replace(' ', r'\ '))[:33] + '$'
 
             for e, var in enumerate(self._algebs):
                 unamey = self._unamey[e]
                 fnamey = self._fnamey[e]
                 idx = self.__dict__[var][i]
 
-                varname.unamey[idx] = '{} {}'.format(unamey, iname)[:24]
-                varname.fnamey[idx] = '$' + '{}\ {}'.format(
-                    fnamey, iname.replace(' ', '\\ '))[:24] + '$'
+                varname.unamey[idx] = '{} {}'.format(unamey, iname)[:33]
+                varname.fnamey[idx] = '$' + r'{}\ {}'.format(
+                    fnamey, iname.replace(' ', r'\ '))[:33] + '$'
 
     def _param_to_matrix(self):
         """
@@ -1208,6 +1318,7 @@ class ModelBase(object):
                 self.__dict__[key][idx] = minval
 
     def __repr__(self):
+
         ret = ''
         ret += '\n'
         ret += 'Model <{:s}> parameters in element base\n'.format(self._name)
@@ -1344,6 +1455,9 @@ class ModelBase(object):
 
             ret.append(idx)
 
+        if len(ret) == 1:
+            ret = ret[0]
+
         return ret
 
     def link_bus(self, bus_idx):
@@ -1433,7 +1547,82 @@ class ModelBase(object):
                         '<{}> has Vdcn={} different from node <{}> Vdcn={}.'
                         .format(name, Vdcn, node, Vdcn0), WARNING)
 
-    #
+    def link_from(self):
+        """
+
+        Returns
+        -------
+
+        """
+        pass
+
+    def link_to(self, model, idx, self_idx):
+        """
+        Register (self.name, self.idx) in `model._from`
+
+        Returns
+        -------
+
+        """
+        if model in self.system.loaded_groups:
+
+            # access group instance
+            grp = self.system.__dict__[model]
+
+            # doing it one by one
+            for i, self_i in zip(idx, self_idx):
+                # query model name and access model instance
+                mdl_name = grp._idx_model[i]
+                mdl = self.system.__dict__[mdl_name]
+
+                # query the corresponding uid
+                u = mdl.get_uid(i)
+
+                # update `mdl_from`
+                name_idx_pair = (self._name, self_i)
+                if name_idx_pair not in mdl.mdl_from[u]:
+                    mdl.mdl_from[u].append(name_idx_pair)
+
+        else:
+            # access model instance
+            mdl = self.system.__dict__[model]
+            uid = mdl.get_uid(idx)
+
+            for u, self_i in zip(uid, self_idx):
+                name_idx_pair = (self._name, self_i)
+
+                if name_idx_pair not in mdl.mdl_from[u]:
+                    mdl.mdl_from[u].append(name_idx_pair)
+
+    def get_element_data(self, idx):
+        """
+        Get all data associated with an element whose index is `idx` in a dict.
+
+        Returns
+        -------
+        dict : param, value pairs
+        """
+        out = {}
+        if idx not in self.idx:
+            logger.debug('Model {} does not have element {}'.format(self.name, idx))
+            return out
+
+        idx_int = self.get_uid(idx)
+
+        out['idx'] = self.idx[idx_int]
+        out['name'] = self.name[idx_int]
+
+        for item in self._data.keys():
+            out[item] = self.__dict__[item][idx_int]
+
+        for item in self._algebs:
+            out[item] = self.__dict__[item][idx_int]
+
+        for item in self._states:
+            out[item] = self.__dict__[item][idx_int]
+
+        return out
+
     # def var_store_snapshot(self, variable='all'):
     #     """
     #     Store a snapshot of variable values to self._snapshot.
