@@ -11,7 +11,7 @@ class PI1(object):
 
     def __init__(self, params=None, inputs=None, outputs=None):
 
-        assert type(outputs) == list
+        assert isinstance(outputs, list)
 
         if hasattr(self, 'nPI1'):
             self.nPI1 += 1
@@ -24,7 +24,7 @@ class PI1(object):
             self.PI1_label = ['PI1' + str(self.nPI1)]
 
         if params:
-            assert type(params) == dict
+            assert isinstance(params, dict)
         else:
             params = {
                 'Kp' + str(self.nPI1): 0.1,
@@ -33,6 +33,6 @@ class PI1(object):
 
         self._algebs = ['']
         if inputs:
-            assert type(inputs) == list
+            assert isinstance(inputs, list)
         else:
             inputs = ['PI_IN_']
