@@ -44,8 +44,8 @@ class VSC(DCBase):
             'vlow': 0.0,
         })
         self._units.update({
-            'rsh': 'omh',
-            'xsh': 'omh',
+            'rsh': 'ohm',
+            'xsh': 'ohm',
             'vshmax': 'pu',
             'vshmin': 'pu',
             'Ishmax': 'pu',
@@ -118,9 +118,6 @@ class VSC(DCBase):
         self.ylim = []
         self.vio = {}
 
-    def base(self):
-        super(VSC, self).base()
-
     def setup(self):
         super().setup()
         self.K = mul(self.K, self.droop)
@@ -182,7 +179,7 @@ class VSC(DCBase):
         vlower = list(abs(Vsh) - self.vshmin)
         iupper = list(abs(IshC) - self.Ishmax)
         # check for Vsh and Ish limit violations
-        if self.system.pflow.config.iter >= self.system.pflow.config.ipv2pq:
+        if self.system.pflow.niter >= self.system.pflow.config.ipv2pq:
             for i in range(self.n):
                 if self.u[i] and (vupper[i] > 0 or vlower[i] < 0
                                   or iupper[i] > 0):
@@ -1077,9 +1074,6 @@ class VSC1(VSC1_Common, VSC1_Outer1, Current1, PLL1, Power0):
         Power0.__init__(self, system, name)
         self._init()
 
-    def base(self):
-        super(VSC1, self).base()
-
 
 class VSC1_IE(VSC1_Common, VSC1_Outer1, Current1, PLL1, Power1):
     """VSC1 with Inertia Emulation with frequency sensor at local bus"""
@@ -1092,9 +1086,6 @@ class VSC1_IE(VSC1_Common, VSC1_Outer1, Current1, PLL1, Power1):
         Power1.__init__(self, system, name)
         self._init()
 
-    def base(self):
-        super(VSC1_IE, self).base()
-
 
 class VSC1_IE2(VSC1_Common, VSC1_Outer1, Current1, PLL1, Power2):
     """VSC1 with Inertia Emulation with frequency sensor at remote bus"""
@@ -1107,9 +1098,6 @@ class VSC1_IE2(VSC1_Common, VSC1_Outer1, Current1, PLL1, Power2):
         Power2.__init__(self, system, name)
         self._init()
 
-    def base(self):
-        super(VSC1_IE2, self).base()
-
 
 class VSC2_Voltage1(object):
     """
