@@ -1,10 +1,14 @@
 import logging
+import importlib
+import scipy.sparse.linalg  # NOQA
+import numpy.linalg
+
 from math import ceil
 
-import numpy.linalg
+from scipy.sparse import csr_matrix  # NOQA
+
 from cvxopt import matrix, spmatrix, mul, div
 from cvxopt.lapack import gesv
-from matplotlib import pyplot as plt
 
 from .base import RoutineBase
 from andes.config.eig import Eig
@@ -47,6 +51,15 @@ class EIG(RoutineBase):
         self.solver.linsolve(system.dae.Gy, Gyx)
 
         self.As = matrix(system.dae.Fx - system.dae.Fy * Gyx)
+
+        # ------------------------------------------------------
+        # TODO: use scipy eigs
+        # self.As = sparse(self.As)
+        # I = np.array(self.As.I).reshape((-1,))
+        # J = np.array(self.As.J).reshape((-1,))
+        # V = np.array(self.As.V).reshape((-1,))
+        # self.As = csr_matrix((V, (I, J)), shape=self.As.size)
+        # ------------------------------------------------------
         return self.As
 
     def calc_eigvals(self):
@@ -58,6 +71,7 @@ class EIG(RoutineBase):
         None
         """
         self.eigs = numpy.linalg.eigvals(self.As)
+        # TODO: use scipy.sparse.linalg.eigs(self.As)
 
         return self.eigs
 
@@ -70,6 +84,8 @@ class EIG(RoutineBase):
 
         """
         mu, N = numpy.linalg.eig(self.As)
+        # TODO: use scipy.sparse.linalg.eigs(self.As)
+
         N = matrix(N)
         n = len(mu)
         idx = range(n)
@@ -101,10 +117,10 @@ class EIG(RoutineBase):
 
         if system.pflow.solved is False:
             logger.warning(
-                'Power flow not solved. Eigenvalue analysis will not continue.')
+                'Power flow not solved. Eig analysis will not continue.')
             return ret
         elif system.dae.n == 0:
-            logger.warning('No dynamic model. Eivgenvalue analysis will not continue.')
+            logger.warning('No dynamic model. Eig analysis will not continue.')
             return ret
 
         t1, s = elapsed()
@@ -120,12 +136,18 @@ class EIG(RoutineBase):
         self.plot_results()
         ret = True
 
-        t2, s = elapsed(t1)
+        _, s = elapsed(t1)
         logger.info('Eigenvalue analysis finished in {:s}.'.format(s))
 
         return ret
 
     def plot_results(self):
+        try:
+            plt = importlib.import_module('matplotlib.pyplot')
+        except ImportError:
+            plt = None
+            logger.warning('Install matplotlib to plot eigenvalue map.')
+            return False
 
         mu_real = self.mu.real()
         mu_imag = self.mu.imag()
