@@ -36,6 +36,7 @@ class PFLOW(RoutineBase):
         self.niter = 0
         self.iter_mis = []
         self.F = None
+        self.system.dae.factorize = True
 
     def pre(self):
         """
@@ -56,13 +57,17 @@ class PFLOW(RoutineBase):
 
         system.dae.init_xy()
 
-        for device, pflow, init0 in zip(system.devman.devices, system.call.pflow, system.call.init0):
+        for device, pflow, init0 in zip(system.devman.devices,
+                                        system.call.pflow, system.call.init0):
             if pflow and init0:
                 system.__dict__[device].init0(dae)
 
         # check for islands
         system.check_islands(show_info=True)
 
+        # reset internal storage
+        self.reset()
+
         t, s = elapsed(t)
         logger.debug('Power flow initialized in {:s}.'.format(s))
 
@@ -72,17 +77,20 @@ class PFLOW(RoutineBase):
 
         Returns
         -------
-        bool:
+        bool
             True for success, False for fail
         """
         ret = None
 
+        # initialization Y matrix and inital guess
         self.pre()
         t, _ = elapsed()
 
         # call solution methods
         if self.config.method == 'NR':
             ret = self.newton()
+        elif self.config.method == 'DCPF':
+            ret = self.dcpf()
         elif self.config.method in ('FDPF', 'FDBX', 'FDXB'):
             ret = self.fdpf()
 
@@ -92,7 +100,8 @@ class PFLOW(RoutineBase):
         if self.solved:
             logger.info(' Solution converged in {} in {} iterations'.format(s, self.niter))
         else:
-            logger.warn(' Solution failed in {} in {} iterations'.format(s, self.niter))
+            logger.warning(' Solution failed in {} in {} iterations'.format(s,
+                           self.niter))
         return ret
 
     def newton(self):
@@ -130,6 +139,48 @@ class PFLOW(RoutineBase):
 
         return self.solved, self.niter
 
+    def dcpf(self):
+        """
+        Calculate linearized power flow
+
+        Returns
+        -------
+        (bool, int)
+            success flag, number of iterations
+        """
+        dae = self.system.dae
+
+        self.system.Bus.init0(dae)
+        self.system.dae.init_g()
+
+        Va0 = self.system.Bus.angle
+        for model, pflow, gcall in zip(self.system.devman.devices, self.system.call.pflow, self.system.call.gcall):
+            if pflow and gcall:
+                self.system.__dict__[model].gcall(dae)
+
+        sw = self.system.SW.a
+        sw.sort(reverse=True)
+        no_sw = self.system.Bus.a[:]
+        no_swv = self.system.Bus.v[:]
+
+        for item in sw:
+            no_sw.pop(item)
+            no_swv.pop(item)
+
+        Bp = self.system.Line.Bp[no_sw, no_sw]
+        p = matrix(self.system.dae.g[no_sw], (no_sw.__len__(), 1))
+        p = p-self.system.Line.Bp[no_sw, sw]*Va0[sw]
+
+        Sp = self.solver.symbolic(Bp)
+        N = self.solver.numeric(Bp, Sp)
+        self.solver.solve(Bp, Sp, N, p)
+        self.system.dae.y[no_sw] = p
+
+        self.solved = True
+        self.niter = 1
+
+        return self.solved, self.niter
+
     def _iter_info(self, niter, level=logging.INFO):
         """
         Log iteration number and mismatch
@@ -153,7 +204,7 @@ class PFLOW(RoutineBase):
         Returns
         -------
         matrix
-            The solution to ``x = -A\b``
+            The solution to ``x = -A\\b``
         """
         system = self.system
         self.newton_call()
@@ -164,8 +215,11 @@ class PFLOW(RoutineBase):
         inc = matrix([system.dae.f, system.dae.g])
 
         if system.dae.factorize:
-            self.F = self.solver.symbolic(A)
-            system.dae.factorize = False
+            try:
+                self.F = self.solver.symbolic(A)
+                system.dae.factorize = False
+            except NotImplementedError:
+                pass
 
         try:
             N = self.solver.numeric(A, self.F)
@@ -176,6 +230,8 @@ class PFLOW(RoutineBase):
         except ArithmeticError:
             logger.warning('Jacobian matrix is singular.')
             system.dae.check_diag(system.dae.Gy, 'unamey')
+        except NotImplementedError:
+            inc = self.solver.linsolve(A, inc)
 
         return -inc
 
@@ -195,14 +251,16 @@ class PFLOW(RoutineBase):
         dae = self.system.dae
 
         system.dae.init_fg()
-
+        system.dae.reset_small_g()
         # evaluate algebraic equation mismatches
-        for model, pflow, gcall in zip(system.devman.devices, system.call.pflow, system.call.gcall):
+        for model, pflow, gcall in zip(system.devman.devices,
+                                       system.call.pflow, system.call.gcall):
             if pflow and gcall:
                 system.__dict__[model].gcall(dae)
 
         # eval differential equations
-        for model, pflow, fcall in zip(system.devman.devices, system.call.pflow, system.call.fcall):
+        for model, pflow, fcall in zip(system.devman.devices,
+                                       system.call.pflow, system.call.fcall):
             if pflow and fcall:
                 system.__dict__[model].fcall(dae)
 
@@ -212,7 +270,8 @@ class PFLOW(RoutineBase):
         if system.dae.factorize:
             system.dae.init_jac0()
             # evaluate constant Jacobian elements
-            for model, pflow, jac0 in zip(system.devman.devices, system.call.pflow, system.call.jac0):
+            for model, pflow, jac0 in zip(system.devman.devices,
+                                          system.call.pflow, system.call.jac0):
                 if pflow and jac0:
                     system.__dict__[model].jac0(dae)
             dae.temp_to_spmatrix('jac0')
@@ -220,12 +279,14 @@ class PFLOW(RoutineBase):
         dae.setup_FxGy()
 
         # evaluate Gy
-        for model, pflow, gycall in zip(system.devman.devices, system.call.pflow, system.call.gycall):
+        for model, pflow, gycall in zip(system.devman.devices,
+                                        system.call.pflow, system.call.gycall):
             if pflow and gycall:
                 system.__dict__[model].gycall(dae)
 
         # evaluate Fx
-        for model, pflow, fxcall in zip(system.devman.devices, system.call.pflow, system.call.fxcall):
+        for model, pflow, fxcall in zip(system.devman.devices,
+                                        system.call.pflow, system.call.fxcall):
             if pflow and fxcall:
                 system.__dict__[model].fxcall(dae)
 
@@ -238,8 +299,10 @@ class PFLOW(RoutineBase):
         """
         Post processing for solved systems.
 
-        Store load, generation data on buses. Store reactive power generation on PVs and slack generators.
+        Store load, generation data on buses.
+        Store reactive power generation on PVs and slack generators.
         Calculate series flows and area flows.
+
         Returns
         -------
         None
@@ -308,17 +371,18 @@ class PFLOW(RoutineBase):
         Bp = system.Line.Bp[no_sw, no_sw]
         Bpp = system.Line.Bpp[no_g, no_g]
 
-        Fp = self.solver.symbolic(Bp)
-        Fpp = self.solver.symbolic(Bpp)
-        Np = self.solver.numeric(Bp, Fp)
-        Npp = self.solver.numeric(Bpp, Fpp)
+        # Fp = self.solver.symbolic(Bp)
+        # Fpp = self.solver.symbolic(Bpp)
+        # Np = self.solver.numeric(Bp, Fp)
+        # Npp = self.solver.numeric(Bpp, Fpp)
         exec(system.call.fdpf)
 
         # main loop
         while error > tol:
             # P-theta
             da = matrix(div(system.dae.g[no_sw], system.dae.y[no_swv]))
-            self.solver.solve(Bp, Fp, Np, da)
+            # self.solver.solve(Bp, Fp, Np, da)
+            da = self.solver.linsolve(Bp, da)
             system.dae.y[no_sw] += da
 
             exec(system.call.fdpf)
@@ -326,7 +390,8 @@ class PFLOW(RoutineBase):
 
             # Q-V
             dV = matrix(div(system.dae.g[no_gv], system.dae.y[no_gv]))
-            self.solver.solve(Bpp, Fpp, Npp, dV)
+            # self.solver.solve(Bpp, Fpp, Npp, dV)
+            dV = self.solver.linsolve(Bpp, dV)
             system.dae.y[no_gv] += dV
 
             exec(system.call.fdpf)
