@@ -1,18 +1,18 @@
 import logging
-from math import isnan
-from time import monotonic as time, sleep
 import importlib
 import sys
+import numpy as np
 
-import progressbar
-from cvxopt import matrix, sparse, spdiag
-
-from .base import RoutineBase
 from andes.config.tds import Tds
 from andes.utils import elapsed
 from andes.utils.math import zeros
 from andes.utils.solver import Solver
 
+from .base import RoutineBase
+
+from cvxopt import matrix, sparse, spdiag
+from time import monotonic as time, sleep
+
 logger = logging.getLogger(__name__)
 __cli__ = 'tds'
 
@@ -28,14 +28,31 @@ class TDS(RoutineBase):
 
         # internal states
         self.F = None
-        self.bar = progressbar.ProgressBar(maxval=100,
-                                           widgets=[
-                                               ' [',
-                                               progressbar.Percentage(),
-                                               progressbar.Bar(),
-                                               progressbar.AdaptiveETA(), '] '
-                                           ])
 
+        self.initialized = False
+        self.switch = False
+        self.next_pc = 0.1
+        self.step = 0
+        self.t = self.config.t0
+        self.h = 0
+        self.headroom = 0
+        self.t_jac = 0
+        self.inc = None
+        self.callpert = None
+        self.solved = False
+        self.fixed_times = []
+        self.convergence = True
+        self.niter = 0
+        self.err = 1
+        self.x0 = None
+        self.y0 = None
+        self.f0 = None
+        self.success = False
+
+    def reset(self):
+        self.F = None
+
+        self.initialized = False
         self.switch = False
         self.next_pc = 0.1
         self.step = 0
@@ -156,7 +173,7 @@ class TDS(RoutineBase):
 
         # reduce time step for fixed_times events
         for fixed_t in self.fixed_times:
-            if (fixed_t > self.t) and (fixed_t < self.t + config.deltat):
+            if (fixed_t > self.t) and (fixed_t <= self.t + config.deltat):
                 config.deltat = fixed_t - self.t
                 self.switch = True
                 break
@@ -172,10 +189,16 @@ class TDS(RoutineBase):
         None
         """
         system = self.system
-        config = self.config
-        dae = self.system.dae
+
+        if self.t != self.config.t0:
+            logger.warning('TDS has been previously run and cannot be re-initialized. Please reload the system.')
+            return False
+
         if system.pflow.solved is False:
-            return
+            logger.info('Attempting to solve power flow before TDS.')
+            solved, niter = system.pflow.run()
+            if not solved:
+                return False
 
         t, s = elapsed()
 
@@ -194,11 +217,6 @@ class TDS(RoutineBase):
             if init1:
                 system.__dict__[device].init1(system.dae)
 
-        # compute line and area flow
-        if config.compute_flows:
-            dae.init_fg()
-            self.compute_flows()  # TODO: move to PowerSystem
-
         t, s = elapsed(t)
 
         if system.dae.n:
@@ -211,6 +229,10 @@ class TDS(RoutineBase):
         system.dae.mu = 1.0
         system.dae.kg = 0.0
 
+        self.initialized = True
+
+        return True
+
     def run(self):
         """
         Run time domain simulation
@@ -220,6 +242,12 @@ class TDS(RoutineBase):
         bool
             Success flag
         """
+
+        # check if initialized
+        if not self.initialized:
+            if self.init() is False:
+                logger.info('Call to TDS initialization failed in `tds.run()`.')
+
         ret = False
         system = self.system
         config = self.config
@@ -232,7 +260,11 @@ class TDS(RoutineBase):
             logger.warning('Power flow not solved. Simulation cannot continue.')
             return ret
         t0, _ = elapsed()
+        t1 = t0
 
+        self.streaming_init()
+
+        logger.info('')
         logger.info('-> Time Domain Simulation: {} method, t={} s'
                     .format(self.config.method, self.config.tf))
 
@@ -241,11 +273,13 @@ class TDS(RoutineBase):
         self.run_step0()
 
         config.qrtstart = time()
-        self.bar.start()
 
         while self.t < config.tf:
-            self.calc_time_step()
             self.check_fixed_times()
+            self.calc_time_step()
+
+            if self.callpert is not None:
+                self.callpert(self.t, self.system)
 
             if self.h == 0:
                 break
@@ -269,24 +303,35 @@ class TDS(RoutineBase):
             self.implicit_step()
 
             if self.convergence is False:
-                self.restore_values()
-                continue
+                try:
+                    self.restore_values()
+                    continue
+                except ValueError:
+                    self.t = config.tf
+                    ret = False
+                    break
 
             self.step += 1
             self.compute_flows()
             system.varout.store(self.t, self.step)
+            self.streaming_step()
 
             # plot variables and display iteration status
             perc = max(min((self.t - config.t0) / (config.tf - config.t0) * 100, 100), 0)
 
-            if self.bar is not None:
-                self.bar.update(perc)
+            # show iteration info every 30 seconds or every 20%
+
+            t2, _ = elapsed(t1)
+            if t2 - t1 >= 30:
+                t1 = t2
+                logger.info(' ({:.0f}%) time = {:.4f}s, step = {}, niter = {}'
+                            .format(100 * self.t / config.tf, self.t, self.step,
+                                    self.niter))
 
             if perc > self.next_pc or self.t == config.tf:
                 self.next_pc += 20
-                if self.bar is None:
-                    logger.info(' ({:.0f}%) time = {:.4f}s, step = {}, niter = {}'
-                                .format(100 * self.t / config.tf, self.t, self.step, self.niter))
+                logger.info(' ({:.0f}%) time = {:.4f}s, step = {}, niter = {}'
+                            .format(100 * self.t / config.tf, self.t, self.step, self.niter))
 
             # compute max rotor angle difference
             # diff_max = anglediff()
@@ -306,9 +351,6 @@ class TDS(RoutineBase):
                     while time() - rt_end < 0:
                         sleep(1e-5)
 
-        if self.bar is not None:
-            self.bar.finish()
-
         if config.qrt:
             logger.debug('RT headroom time: {} s.'.format(str(self.headroom)))
 
@@ -318,6 +360,9 @@ class TDS(RoutineBase):
         else:
             ret = True
 
+        if system.config.dime_enable:
+            system.streaming.finalize()
+
         _, s = elapsed(t0)
 
         if ret is True:
@@ -326,6 +371,7 @@ class TDS(RoutineBase):
             logger.info(' Time domain simulation failed in {:s}.'.format(s))
 
         self.success = ret
+        self.dump_results(success=self.success)
 
         return ret
 
@@ -407,8 +453,12 @@ class TDS(RoutineBase):
             dae.reset_Ac()
 
             if dae.factorize:
-                self.F = self.solver.symbolic(dae.Ac)
-                dae.factorize = False
+                try:
+                    self.F = self.solver.symbolic(dae.Ac)
+                    dae.factorize = False
+                except NotImplementedError:
+                    pass
+
             self.inc = -matrix([dae.q, dae.g])
 
             try:
@@ -425,14 +475,16 @@ class TDS(RoutineBase):
                 logger.warning('Unexpected symbolic factorization')
                 dae.factorize = True
                 continue
-            else:
-                inc_x = self.inc[:dae.n]
-                inc_y = self.inc[dae.n:dae.m + dae.n]
-                dae.x += inc_x
-                dae.y += inc_y
+            except NotImplementedError:
+                self.inc = self.solver.linsolve(dae.Ac, self.inc)
+
+            inc_x = self.inc[:dae.n]
+            inc_y = self.inc[dae.n:dae.m + dae.n]
+            dae.x += inc_x
+            dae.y += inc_y
 
             self.err = max(abs(self.inc))
-            if isnan(config.error):
+            if np.isnan(self.inc).any():
                 logger.error('Iteration error: NaN detected.')
                 self.niter = config.maxit + 1
                 break
@@ -499,10 +551,49 @@ class TDS(RoutineBase):
         """
         dae = self.system.dae
         system = self.system
+        config = self.config
+
+        # compute line and area flow
+        if config.compute_flows:
+            dae.init_fg()
+            self.compute_flows()  # TODO: move to PowerSystem
 
         self.inc = zeros(dae.m + dae.n, 1)
         system.varout.store(self.t, self.step)
 
+        self.streaming_step()
+
+    def streaming_step(self):
+        """
+        Sync, handle and streaming for each integration step
+
+        Returns
+        -------
+        None
+        """
+        system = self.system
+        if system.config.dime_enable:
+            system.streaming.sync_and_handle()
+            system.streaming.vars_to_modules()
+            system.streaming.vars_to_pmu()
+
+    def streaming_init(self):
+        """
+        Send out initialization variables and process init from modules
+
+        Returns
+        -------
+        None
+        """
+        system = self.system
+        config = self.config
+        if system.config.dime_enable:
+            config.compute_flows = True
+            system.streaming.send_init(recepient='all')
+            logger.info('Waiting for modules to send init info...')
+            sleep(0.5)
+            system.streaming.sync_and_handle()
+
     def angle_diff(self):
         """
         Compute the maximum angle difference between generators
@@ -535,11 +626,12 @@ class TDS(RoutineBase):
             exec(system.call.seriesflow)
             system.Area.seriesflow(system.dae)
             system.Area.interchange_varout()
+
             dae.y = matrix([
-                dae.y, bus_inj, system.Line._line_flows, system.Area.inter_varout
+                dae.y[:system.dae.m], bus_inj, system.Line._line_flows, system.Area.inter_varout
             ])
 
-    def dump_results(self):
+    def dump_results(self, success):
         """
         Dump simulation results to ``dat`` and ``lst`` files
 
@@ -551,7 +643,9 @@ class TDS(RoutineBase):
 
         t, _ = elapsed()
 
-        if self.success and (not system.files.no_output):
-            system.varout.dump()
+        if success and (not system.files.no_output):
+
+            # system.varout.dump()
+            system.varout.dump_np_vars()
             _, s = elapsed(t)
             logger.info('Simulation data dumped in {:s}.'.format(s))
