@@ -26,12 +26,17 @@ from operator import itemgetter
 
 from . import routines
 from .config import System
-from .consts import pi
-from .consts import rad2deg
+from .consts import pi, rad2deg
 from .models import non_jits, jits, JIT
 from .utils import get_config_load_path
 from .variables import FileMan, DevMan, DAE, VarName, VarOut, Call, Report
 
+try:
+    from andes_addon.streaming import Streaming
+    STREAMING = True
+except ImportError:
+    STREAMING = False
+
 logger = logging.getLogger(__name__)
 
 
@@ -50,9 +55,12 @@ class PowerSystem(object):
                  dynfile=None,
                  addfile=None,
                  config=None,
+                 input_path=None,
                  input_format=None,
                  output_format=None,
+                 output_path='',
                  gis=None,
+                 dime=None,
                  tf=None,
                  **kwargs):
         """
@@ -93,6 +101,9 @@ class PowerSystem(object):
         gis : None or str, optional
             Path to the GIS file
 
+        dime : None or str, optional
+            DiME server address
+
         tf : None or float, optional
             Time-domain simulation end time
 
@@ -105,17 +116,22 @@ class PowerSystem(object):
                              input_format=input_format,
                              addfile=addfile,
                              config=config,
+                             input_path=input_path,
                              no_output=no_output,
                              dynfile=dynfile,
                              dump_raw=dump_raw,
+                             output_path=output_path,
                              output_format=output_format,
                              output=output,
                              **kwargs)
 
         self.config = System()
-        self.routine_import()
 
-        self.load_config(get_config_load_path(self.files.config))
+        conf_path = get_config_load_path(self.files.config)
+        self.load_config(conf_path=conf_path, sys_only=True)
+
+        self.routine_import()
+        self.load_config(conf_path=conf_path, routine_only=True)
 
         self.devman = DevMan(self)
         self.call = Call(self)
@@ -124,9 +140,25 @@ class PowerSystem(object):
         self.varout = VarOut(self)
         self.report = Report(self)
 
+        self.loaded_groups = []
+
+        if dime:
+            self.config.dime_enable = True
+            self.config.dime_server = dime
+
         if tf:
             self.tds.config.tf = tf
 
+        self.streaming = None
+        if not STREAMING:
+            if self.config.dime_enable:
+                self.config.dime_enable = False
+                logger.warning('Missing andes_addon for DiME streaming.')
+        else:
+            if self.config.dime_enable:
+                logger.info('Connecting to DiME at {}'.format(self.config.dime_server))
+                self.streaming = Streaming(self)
+
         self.model_import()
 
     @property
@@ -194,6 +226,19 @@ class PowerSystem(object):
             for item in self.devman.devices:
                 self.__dict__[item].data_to_sys_base()
 
+    def to_elembase(self):
+        """
+        Convert parameters back to element base. This function calls the
+        ```data_to_elem_base``` function.
+
+        Returns
+        -------
+        None
+        """
+        if self.config.base:
+            for item in self.devman.devices:
+                self.__dict__[item].data_to_elem_base()
+
     def group_add(self, name='Ungrouped'):
         """
         Dynamically add a group instance to the system if not exist.
@@ -209,6 +254,7 @@ class PowerSystem(object):
         """
         if not hasattr(self, name):
             self.__dict__[name] = Group(self, name)
+            self.loaded_groups.append(name)
 
     def model_import(self):
         """
@@ -390,7 +436,7 @@ class PowerSystem(object):
 
         return times
 
-    def load_config(self, conf_path):
+    def load_config(self, conf_path, sys_only=False, routine_only=False):
         """
         Load config from an ``andes.conf`` file.
 
@@ -413,10 +459,12 @@ class PowerSystem(object):
 
         conf = configparser.ConfigParser()
         conf.read(conf_path)
-
-        self.config.load_config(conf)
-        for r in routines.__all__:
-            self.__dict__[r.lower()].config.load_config(conf)
+        if sys_only:
+            self.config.load_config(conf)
+            return
+        if routine_only:
+            for r in routines.__all__:
+                self.__dict__[r.lower()].config.load_config(conf)
 
         logger.debug('Loaded config file from {}.'.format(conf_path))
 
@@ -501,12 +549,12 @@ class PowerSystem(object):
                 logger.warning(
                     'Multiple slack buses are defined for {:g} island(s).'.
                     format(len(nosw_island)))
-            else:
-                logger.debug(
-                    'Each island has a slack bus correctly defined.'.format(
-                        nosw_island))
 
-    def get_busdata(self, dec=5):
+            if (not nosw_island) and (not msw_island):
+                logger.debug(
+                    'Each island has a slack bus correctly defined.')
+
+    def get_busdata(self, sort_names=False):
         """
         get ac bus data from solved power flow
         """
@@ -525,10 +573,16 @@ class PowerSystem(object):
         Qg = [self.Bus.Qg[x] for x in range(self.Bus.n)]
         Pl = [self.Bus.Pl[x] for x in range(self.Bus.n)]
         Ql = [self.Bus.Ql[x] for x in range(self.Bus.n)]
-        return (list(x) for x in zip(*sorted(
-            zip(idx, names, Vm, Va, Pg, Qg, Pl, Ql), key=itemgetter(0))))
 
-    def get_nodedata(self, dec=5):
+        if sort_names:
+            ret = (list(x) for x in zip(*sorted(
+                zip(idx, names, Vm, Va, Pg, Qg, Pl, Ql), key=itemgetter(0))))
+        else:
+            ret = idx, names, Vm, Va, Pg, Qg, Pl, Ql
+
+        return ret
+
+    def get_nodedata(self, sort_names=False):
         """
         get dc node data from solved power flow
         """
@@ -540,26 +594,44 @@ class PowerSystem(object):
         idx = self.Node.idx
         names = self.Node.name
         V = [self.dae.y[x] for x in self.Node.v]
-        return (list(x)
-                for x in zip(*sorted(zip(idx, names, V), key=itemgetter(0))))
 
-    def get_linedata(self, dec=5):
-        """get line data from solved power flow"""
+        if sort_names:
+            ret = (list(x)
+                   for x in zip(*sorted(zip(idx, names, V), key=itemgetter(0))))
+        else:
+            ret = idx, names, V
+
+        return ret
+
+    def get_linedata(self, sort_names=False):
+        """
+        get line data from solved power flow
+        """
         if not self.pflow.solved:
             logger.error('Power flow not solved when getting line data.')
             return tuple([False] * 7)
         idx = self.Line.idx
         fr = self.Line.bus1
         to = self.Line.bus2
-        Pfr = [self.Line.S1[x].real for x in range(self.Line.n)]
-        Qfr = [self.Line.S1[x].imag for x in range(self.Line.n)]
-        Pto = [self.Line.S2[x].real for x in range(self.Line.n)]
-        Qto = [self.Line.S2[x].imag for x in range(self.Line.n)]
-        Ploss = [i + j for i, j in zip(Pfr, Pto)]
-        Qloss = [i + j for i, j in zip(Qfr, Qto)]
-        return (list(x) for x in zip(*sorted(
-            zip(idx, fr, to, Pfr, Qfr, Pto, Qto, Ploss, Qloss),
-            key=itemgetter(0))))
+
+        Sloss = self.Line.S1 + self.Line.S2
+
+        Pfr = list(self.Line.S1.real())
+        Qfr = list(self.Line.S1.imag())
+        Pto = list(self.Line.S2.real())
+        Qto = list(self.Line.S2.imag())
+
+        Ploss = list(Sloss.real())
+        Qloss = list(Sloss.imag())
+
+        if sort_names:
+            ret = (list(x) for x in zip(*sorted(
+                zip(idx, fr, to, Pfr, Qfr, Pto, Qto, Ploss, Qloss),
+                key=itemgetter(0))))
+        else:
+            ret = idx, fr, to, Pfr, Qfr, Pto, Qto, Ploss, Qloss
+
+        return ret
 
 
 class GroupMeta(type):
@@ -579,6 +651,7 @@ class Group(metaclass=GroupMeta):
         self.name = name
         self.all_models = []
         self._idx_model = {}
+        self._idx = []
 
     def register_model(self, model):
         """
@@ -604,9 +677,9 @@ class Group(metaclass=GroupMeta):
         if idx is None:
             idx = model + '_' + str(len(self._idx_model))
 
-        assert idx not in self._idx_model.values()
-
         self._idx_model[idx] = model
+        self._idx.append(idx)
+
         return idx
 
     def get_field(self, field, idx):
@@ -620,6 +693,8 @@ class Group(metaclass=GroupMeta):
         ret = []
         scalar = False
 
+        # TODO: ensure idx is unique in this Group
+
         if isinstance(idx, (int, float, str)):
             scalar = True
             idx = [idx]
