@@ -4,6 +4,7 @@ from ..utils.math import ageb, aleb, aandb, agtb  # NOQA
 from ..utils.math import index, altb  # NOQA
 import numpy as np
 import logging
+import scipy.sparse as sps  # NOQA
 
 logger = logging.getLogger(__name__)
 
@@ -47,58 +48,6 @@ class DAE(object):
         self.__dict__.update(self._scalars)
         self.__dict__.update(self._flags)
 
-        # self._temp = {'Fx': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #               'Fy': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #               'Gx': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #               'Gy': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #               'Fx0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                       'J': np.ndarray((0, 1), dtype=np.int32),
-        #                       'V': np.ndarray((0, 1))},
-        #               'Fy0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                       'J': np.ndarray((0, 1), dtype=np.int32),
-        #                       'V': np.ndarray((0, 1))},
-        #               'Gx0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                       'J': np.ndarray((0, 1), dtype=np.int32),
-        #                       'V': np.ndarray((0, 1))},
-        #               'Gy0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                       'J': np.ndarray((0, 1), dtype=np.int32),
-        #                       'V': np.ndarray((0, 1))},
-        #               }
-        #
-        # self._set = {'Fx': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                     'J': np.ndarray((0, 1), dtype=np.int32),
-        #                     'V': np.ndarray((0, 1))},
-        #              'Fy': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                     'J': np.ndarray((0, 1), dtype=np.int32),
-        #                     'V': np.ndarray((0, 1))},
-        #              'Gx': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                     'J': np.ndarray((0, 1), dtype=np.int32),
-        #                     'V': np.ndarray((0, 1))},
-        #              'Gy': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                     'J': np.ndarray((0, 1), dtype=np.int32),
-        #                     'V': np.ndarray((0, 1))},
-        #              'Fx0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #              'Fy0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #              'Gx0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #              'Gy0': {'I': np.ndarray((0, 1), dtype=np.int32),
-        #                      'J': np.ndarray((0, 1), dtype=np.int32),
-        #                      'V': np.ndarray((0, 1))},
-        #              }
-
         self._temp = {
             'Fx': {
                 'I': matrix([]),
@@ -358,6 +307,7 @@ class DAE(object):
         """
         yext = self.m - len(self.y)
         xext = self.n - len(self.x)
+
         if yext > 0:
             yzeros = zeros(yext, 1)
             yones = ones(yext, 1)
@@ -366,6 +316,9 @@ class DAE(object):
             self.uy = matrix([self.uy, yones], (self.m, 1), 'd')
             self.zymin = matrix([self.zymin, yones], (self.m, 1), 'd')
             self.zymax = matrix([self.zymax, yones], (self.m, 1), 'd')
+        else:
+            logger.info('Skip extending DAE algeb arrays. Did you call `tds.init()` more than once or '
+                        'have no dynamic components?')
         if xext > 0:
             xzeros = zeros(xext, 1)
             xones = ones(xext, 1)
@@ -374,6 +327,9 @@ class DAE(object):
             self.ux = matrix([self.ux, xones], (self.n, 1), 'd')
             self.zxmin = matrix([self.zxmin, xones], (self.n, 1), 'd')
             self.zxmax = matrix([self.zxmax, xones], (self.n, 1), 'd')
+        else:
+            logger.info('Skip extending DAE state arrays. Did you call `tds.init()` more than once or '
+                        'have no dynamic components?')
 
     def hard_limit(self, yidx, ymin, ymax, min_set=None, max_set=None):
         """Set hard limits for algebraic variables and reset the equation mismatches
@@ -434,17 +390,10 @@ class DAE(object):
             self.g[yidx[idx]] = 0
             self.ac_reset = True
 
-    def hard_limit_remote(self,
-                          yidx,
-                          ridx,
-                          rtype='y',
-                          rmin=None,
-                          rmax=None,
-                          min_yset=0,
-                          max_yset=0):
+    def hard_limit_remote(self, yidx, ridx, rtype='y', rmin=None, rmax=None, min_yset=0, max_yset=0):
         """Limit the output of yidx if the remote y is not within the limits
 
-        This function needs to be modernized.
+        TODO: This function needs to be refactored for clearliness.
         """
         ny = len(yidx)
         assert ny == len(
@@ -537,13 +486,14 @@ class DAE(object):
 
         :return: None
         """
-        if self.ac_reset is False and not self.rebuild:
+        if self.ac_reset is False:
             return
 
         mn = self.m + self.n
 
         x = index(aandb(self.zxmin, self.zxmax), 0.)
         y = [i + self.n for i in index(aandb(self.zymin, self.zymax), 0.)]
+
         xy = list(x) + y
 
         eye = spdiag([1.0] * mn)
